@@ -44,9 +44,6 @@ except ImportError:
         except ImportError:
             import json as jsonlib
 
-import logging
-
-
 class TropoAction(object):
     """
     Class representing the base Tropo action.
@@ -91,7 +88,8 @@ class Ask(TropoAction):
         if (isinstance(choices, basestring)):
             self._dict['choices'] = Choices(choices).json
         else:
-            self._dict['choices'] = choices['choices']
+#            self._dict['choices'] = choices['choices']
+            self._dict['choices'] = choices.json
         for opt in self.options_array:
             if opt in options:
                 if ((opt == 'say') and (isinstance(options['say'], basestring))):
@@ -121,13 +119,18 @@ class Call(TropoAction):
         "timeout": Float } }
     """
     action = 'call'
-    options_array = ['answerOnMedia', 'channel', 'from', 'headers', 'name', 'network', 'recording', 'required', 'timeout', 'allowSignals']
+    options_array = ['answerOnMedia', 'channel', '_from', 'headers', 'name', 'network', 'recording', 'required', 'timeout', 'allowSignals']
 
     def __init__(self, to, **options):
         self._dict = {'to': to}
         for opt in self.options_array:
             if opt in options:
-                self._dict[opt] = options[opt]
+                if (opt == "_from"):
+                    self._dict['from'] = options[opt]
+                else:
+                    self._dict[opt] = options[opt]
+
+                
 
 class Choices(TropoAction):
     """
@@ -201,7 +204,7 @@ class Message(TropoAction):
             "to": String or Array,#Required
             "answerOnMedia": Boolean,
             "channel": string,
-            "from": Object,
+            "from": String,
             "name": String,
             "network": String,
             "required": Boolean,
@@ -209,13 +212,17 @@ class Message(TropoAction):
             "voice": String } }
     """
     action = 'message'
-    options_array = ['answerOnMedia', 'channel', 'from', 'name', 'network', 'required', 'timeout', 'voice']
+    options_array = ['answerOnMedia', 'channel', '_from', 'name', 'network', 'required', 'timeout', 'voice']
 
     def __init__(self, say_obj, to, **options):
         self._dict = {'say': say_obj['say'], 'to': to}
         for opt in self.options_array:
             if opt in options:
-                self._dict[opt] = options[opt]
+                if (opt == "_from"):
+                    self._dict['from'] = options[opt]
+                else:
+                    self._dict[opt] = options[opt]
+
 
 class On(TropoAction):
     """
@@ -271,10 +278,11 @@ class Record(TropoAction):
             "timeout": Float,
             "transcription": Array or Object,
             "url": String,#Required ?????
-            "username": String } }
+            "username": String,
+            "voice": String} }
     """
     action = 'record'
-    options_array = ['attempts', 'bargein', 'beep', 'choices', 'format', 'maxSilence', 'maxTime', 'method', 'minConfidence', 'name', 'password', 'required', 'say', 'timeout', 'transcription', 'url', 'username', 'allowSignals']
+    options_array = ['attempts', 'bargein', 'beep', 'choices', 'format', 'maxSilence', 'maxTime', 'method', 'minConfidence', 'name', 'password', 'required', 'say', 'timeout', 'transcription', 'url', 'username', 'allowSignals', 'voice']
 
     def __init__(self, **options):
         self._dict = {}
@@ -342,13 +350,17 @@ class Say(TropoAction):
         } }
     """
     action = 'say'
-    options_array = ['as', 'name', 'required', 'voice', 'allowSignals']
+    # added _as because 'as' is reserved
+    options_array = ['_as', 'name', 'required', 'voice', 'allowSignals']
 
     def __init__(self, message, **options):
         dict = {}
         for opt in self.options_array:
             if opt in options:
-                dict[opt] = options[opt]
+                if (opt == "_as"):
+                    dict['as'] = options['_as']
+                else:
+                    dict[opt] = options[opt]
         self._list = []
         if (isinstance (message, list)):
             for mess in message:
@@ -419,21 +431,25 @@ class Transfer(TropoAction):
         "to": String or Array,#Required
         "answerOnMedia": Boolean,
         "choices": Object,
-        "from": Object,
+	# # **Wed May 18 21:14:05 2011** -- egilchri
+	"headers": Object,
+	# # **Wed May 18 21:14:05 2011** -- egilchri
+	
+        "from": String,
         "name": String,
         "required": Boolean,
         "terminator": String,
         "timeout": Float } }
     """
     action = 'transfer'
-    options_array = ['answerOnMedia', 'choices', 'from', 'name', 'on', 'required', 'allowSignals']
+    options_array = ['answerOnMedia', 'choices', '_from', 'name', 'on', 'required', 'allowSignals', 'headers']
 
     def __init__(self, to, **options):
         self._dict = {'to': to}
         for opt in self.options_array:
             if opt in options:
-                if (opt == 'from'):
-                    self._dict['from'] = options['from']
+                if (opt == '_from'):
+                    self._dict['from'] = options['_from']
                 elif(opt == 'choices'):
                     self._dict['choices'] = options['choices']
                 else:
@@ -458,7 +474,6 @@ class Result(object):
     options_array = ['actions','complete','error','sequence', 'sessionDuration', 'sessionId', 'state']
 
     def __init__(self, result_json):
-        logging.info ("result POST data: %s" % result_json)
         result_data = jsonlib.loads(result_json)
         result_dict = result_data['result']
 
@@ -473,27 +488,42 @@ class Result(object):
         actions = self._actions
 
         if (type (actions) is list):
-            logging.info ("Actions is a list")
             dict = actions[0]
         else:
-            logging.info ("Actions is a dict")
             dict = actions
-        logging.info ("Actions is: %s" % actions)
+        # return dict['value'] Fixes issue 17
+        return dict['value']
+
+# # **Tue May 17 07:17:38 2011** -- egilchri
+
+    def getInterpretation(self):
+        """
+        Get the value of the previously POSTed Tropo action.
+        """
+        actions = self._actions
+
+        if (type (actions) is list):
+            dict = actions[0]
+        else:
+            dict = actions
         return dict['interpretation']
 
+# # **Tue May 17 07:17:38 2011** -- egilchri
+
 
 class Session(object):
     """
     Session is the payload sent as an HTTP POST to your web application when a new session arrives.
     (See https://www.tropo.com/docs/webapi/session.htm)
+    
+    Because 'from' is a reserved word in Python, the session object's 'from' property is called
+    fromaddress in the Python library
     """
     def __init__(self, session_json):
-        logging.info ("POST data: %s" % session_json)
         session_data = jsonlib.loads(session_json)
         session_dict = session_data['session']
         for key in session_dict:
             val = session_dict[key]
-            logging.info ("key: %s val: %s" % (key, val))
             if key == "from":
                 setattr(self, "fromaddress", val)
             else:
@@ -516,14 +546,32 @@ class Tropo(object):
     def  __init__(self):
         self._steps = []
 
+# # **Sun May 15 21:05:01 2011** -- egilchri
+    def setVoice(self, voice):
+        self.voice = voice
+
+# # end **Sun May 15 21:05:01 2011** -- egilchri
+
     def ask(self, choices, **options):
         """
 	 Sends a prompt to the user and optionally waits for a response.
          Arguments: "choices" is a Choices object
          See https://www.tropo.com/docs/webapi/ask.htm
         """
+# # **Sun May 15 21:21:29 2011** -- egilchri
+
+        # Settng the voice in this method call has priority.
+	# Otherwise, we can pick up the voice from the Tropo object,
+	# if it is set there.
+        if hasattr (self, 'voice'):
+            if (not 'voice' in options):
+                options['voice'] = self.voice
+        
+# # **Sun May 15 21:21:29 2011** -- egilchri
+
         self._steps.append(Ask(choices, **options).obj)
 
+
     def call (self, to, **options):
         """
 	 Places a call or sends an an IM, Twitter, or SMS message. To start a call, use the Session API to tell Tropo to launch your code.
@@ -608,6 +656,17 @@ class Tropo(object):
         Argument: **options is a set of optional keyword arguments.
         See https://www.tropo.com/docs/webapi/say.htm
         """
+        #voice = self.voice
+# # **Sun May 15 21:21:29 2011** -- egilchri
+
+        # Settng the voice in this method call has priority.
+	# Otherwise, we can pick up the voice from the Tropo object,
+	# if it is set there.
+        if hasattr (self, 'voice'):
+            if (not 'voice' in options):
+                options['voice'] = self.voice
+# # **Sun May 15 21:21:29 2011** -- egilchri
+
         self._steps.append(Say(message, **options).obj)
 
     def startRecording(self, url, **options):
@@ -642,7 +701,6 @@ class Tropo(object):
         steps = self._steps
         topdict = {}
         topdict['tropo'] = steps
-        logging.info ("topdict: %s" % topdict)
         if pretty:
             try:
                 json = jsonlib.dumps(topdict, indent=4, sort_keys=False)
